#include "msp430fr5739.h"
#include "derection.h"

extern volatile unsigned int ADCResult;
extern volatile unsigned char active; 
extern volatile unsigned char ULP;
extern volatile unsigned char mode;
extern volatile unsigned char UserInput;

volatile unsigned char TX_Buffer[7] = {0,0,0,0,0,0,0};
volatile unsigned char ThreshRange[3]={0,0,0};
unsigned char counter =0;
unsigned int WriteCounter = 0;
unsigned int LEDCounter=0;
unsigned char ULPBreakSync =0;
unsigned int ModeAddress=0;
unsigned int CalValue =0;
unsigned int ADCTemp =0;
unsigned char temp = 0;
unsigned int  si=8;
  int angel=0;

/**********************************************************************//**
 * @brief  Long Delay
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void LongDelay()
{
  __delay_cycles(250000);
  __no_operation();
}



/**********************************************************************//**
 * @brief  Initializes system 
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void SystemInit(void)
{
  // Startup clock system in max. DCO setting ~8MHz
  // This value is closer to 10MHz on untrimmed parts 
  CSCTL0_H = 0xA5;                          // Unlock register
  CSCTL1 |= DCOFSEL0 + DCOFSEL1;            // Set max. DCO setting
  CSCTL2 = SELA_1 + SELS_3 + SELM_3;        // set ACLK = vlo; MCLK = DCO
  CSCTL3 = DIVA_0 + DIVS_0 + DIVM_0;        // set all dividers 
  CSCTL0_H = 0x01;                          // Lock Register
  
    // Turn off temp. 
  REFCTL0 |= REFTCOFF; 
  REFCTL0 &= ~REFON;  
  
  // Enable switches
  // P4.0 and P4.1 are configured as switches
  // Port 4 has only two pins    
/*  P4OUT |= BIT0 +BIT1;                      // Configure pullup resistor  
  P4DIR &= ~(BIT0 + BIT1);                  // Direction = input
  P4REN |= BIT0 + BIT1;                     // Enable pullup resistor
  P4IES &= ~(BIT0+BIT1);                    // P4.0 Lo/Hi edge interrupt  
  P4IE = BIT0+BIT1;                         // P4.0 interrupt enabled
  P4IFG = 0;                                // P4 IFG cleared
 */ 
   // Enable LEDs
  P3OUT &= ~(BIT6+BIT7+BIT5+BIT4);
  P3DIR |= BIT6+BIT7+BIT5+BIT4;
  PJOUT &= ~(BIT0+BIT1+BIT2+BIT3);
  PJDIR |= BIT0 +BIT1+BIT2+BIT3;
  
    // Configure UART pins P2.0 & P2.1
  P2SEL1 |= BIT0 + BIT1;
  P2SEL0 &= ~(BIT0 + BIT1);
  
   // P3.0,P3.1 and P3.2 are accelerometer inputs
  P3OUT &= ~(BIT0 + BIT1 + BIT2);   
  P3DIR &= ~(BIT0 + BIT1 + BIT2); 
  P3REN |= BIT0 + BIT1 + BIT2;
  
}



/**********************************************************************//**
 * @brief  Startup LED sequence
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void StartUpSequence(void) 
{
  unsigned char flag=4,up=1,counter = 0; 
  unsigned char LED_ArrayPJ[] = {0x01,0x02,0x04,0x08};
  unsigned char LED_ArrayP3[] = {0x80,0x40,0x20,0x10};

  while (counter <10) 
    {    
      counter++;
      PJOUT &= ~(BIT0 +BIT1+BIT2+BIT3);
      P3OUT &= ~(BIT4 +BIT5+BIT6+BIT7);    
      if(up)// from the outside - in
        {
          while(flag)
            {
              P3OUT = LED_ArrayP3[flag-1];
              PJOUT = LED_ArrayPJ[flag-1];
              LongDelay();               
              flag--;
            }
            up=0;
        }
      else
        {
          while(flag<4)
            {
              P3OUT = LED_ArrayP3[flag];
              PJOUT = LED_ArrayPJ[flag];
              LongDelay();        
              flag++;
            }
            up = 1;
          }
    } 
  
  // Exit Loop, Clear LEDs
  PJOUT &= ~(BIT0 +BIT1+BIT2+BIT3);
  P3OUT &= ~(BIT4 +BIT5+BIT6+BIT7);    
}


/**********************************************************************//**
 * @brief  Initializes Accelerometer
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void SetupAccel(void)
{  
  //Setup  accelerometer
  // ~20KHz sampling
  //Configure GPIO
  ACC_PORT_SEL0 |= ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN;    //Enable A/D channel inputs
  ACC_PORT_SEL1 |= ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN;
  ACC_PORT_DIR &= ~(ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN);
  ACC_PWR_PORT_DIR |= ACC_PWR_PIN;              //Enable ACC_POWER
  ACC_PWR_PORT_OUT |= ACC_PWR_PIN;

  // Allow the accelerometer to settle before sampling any data 
  __delay_cycles(200000);   
  
  //Single channel, once, 
  ADC10CTL0 &= ~ADC10ENC;                        // Ensure ENC is clear
  ADC10CTL0 = ADC10ON + ADC10SHT_5+ADC10MSC;  
  ADC10CTL1 = ADC10SHS_0 + ADC10SHP + ADC10CONSEQ_1 + ADC10SSEL_0;  
  ADC10CTL2 = ADC10RES;    
  ADC10MCTL0 = ADC10SREF_0 + ADC10INCH_12+ADC10INCH_13+ADC10INCH_14;
  ADC10IV = 0x00;                          // Clear all ADC12 channel int flags  
  ADC10IE |= ADC10IE0;
  
  // Setup Thresholds for relative difference in accelerometer measurements
  ThreshRange[0]=25;
  ThreshRange[1]=50;
  ThreshRange[2]=75;
}


/**********************************************************************//**
 * @brief  ShutDownAccel
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void ShutDownAccel(void)
{  
  ACC_PORT_SEL0 &= ~(ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN);
  ACC_PORT_SEL1 &= ~(ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN);
  ACC_PORT_DIR &= ~(ACC_X_PIN + ACC_Y_PIN + ACC_Z_PIN);
  ACC_PWR_PORT_DIR &= ~ACC_PWR_PIN;            
  ACC_PWR_PORT_OUT &= ~ACC_PWR_PIN;
  ADC10CTL0 &= ~(ADC10ENC + ADC10ON);
  ADC10IE &= ~ADC10IE0;
  ADC10IFG = 0;
}


/**********************************************************************//**
 * @brief  Calibrate thermistor or accelerometer
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
unsigned int CalibrateADC(void)
{
  unsigned char CalibCounter =0;
  unsigned int Value = 0;

  // Disable interrupts & user input during calibration
  //DisableSwitches();            
  while(CalibCounter <50)
    {
      P3OUT ^= BIT4;
      CalibCounter++;
      while (ADC10CTL1 & BUSY); 
      ADC10CTL0 |= ADC10ENC | ADC10SC ;       // Start conversion 
      __bis_SR_register(CPUOFF + GIE);        // LPM0, ADC10_ISR will force exit
      __no_operation(); 
      Value += ADCResult;
    }
    Value = Value/50;
    // Reenable switches after calibration
    //EnableSwitches();
    return Value;
}

/**********************************************************************//**
 * @brief  Disables Switches
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void DisableSwitches(void)
{
  // disable switches
  P4IFG = 0;                                // P4 IFG cleared    
  P4IE &= ~(BIT0+BIT1);                     // P4.0 interrupt disabled
  P4IFG = 0;                                // P4 IFG cleared  
}


/**********************************************************************//**
 * @brief  Enables Switches 
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void EnableSwitches(void)

{
  P4IFG = 0;                                // P4 IFG cleared  
  P4IE = BIT0+BIT1;                         // P4.0 interrupt enabled
}

/**********************************************************************//**
 * @brief  Take ADC Measurement
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void TakeADCMeas(void)
{  
  while (ADC10CTL1 & BUSY); 
  ADC10CTL0 |= ADC10ENC | ADC10SC ;       // Start conversion 
  __bis_SR_register(CPUOFF + GIE);        // LPM0, ADC10_ISR will force exit
  __no_operation();                       // For debug only
}

/**********************************************************************//**
 * @brief  LED Toggle Sequence
 * 
 * @param  
 * DiffValue Difference between calibrated and current measurement
 * temp Direction of difference (positive or negative)
 * @return none
 *************************************************************************/
void LEDSequence(unsigned int DiffValue, unsigned char temp)
{  
  
  // The same scale is used for cold & hot and tilt up/down
  // only the thresholds are different
  P3OUT |= BIT4;                            // Light up the middle LEDs
  PJOUT |= BIT3;
  
  if(DiffValue < ThreshRange[0])            // Very close to CAL value
    {
      P3OUT |= BIT4;                          
      PJOUT |= BIT3;
      PJOUT &= ~(BIT0+BIT1+BIT2);
      P3OUT &= ~(BIT7+BIT6+BIT5);
      counter = 0x34;
 

    }
  
  if ((DiffValue >=ThreshRange[0]) && (DiffValue < ThreshRange[1]))
    {
      // Light up one LED  	
      if(temp == UP)                        // Tilt up, temp up
        {
          PJOUT |= BIT2;
          PJOUT &= ~(BIT1+BIT0);
          P3OUT &= ~(BIT7+BIT6+BIT5);
          counter = 5;
      

        }
      else                                  // Tilt down, temp down
        {
          PJOUT &= ~(BIT0+BIT1+BIT2);
          P3OUT |= BIT5; 
          P3OUT &= ~(BIT6+BIT7);
          counter = 2;
         ;

        }
    }
  if ((DiffValue >= ThreshRange[1]) && (DiffValue < ThreshRange[2]))
    {
      // Light up two LEDs  	    
      if(temp == UP)                        // Tilt up 2, temp up 2
        {
          PJOUT |= BIT2 + BIT1;
          PJOUT &= ~(BIT0);
          P3OUT &= ~(BIT7+BIT6+BIT5);
          counter = 6;
         
        }
      else                                  // Tilt down 2, temp down 2
        {
          PJOUT &= ~(BIT2+BIT1+BIT0);
          P3OUT |= BIT5 + BIT6; 
          P3OUT &= ~(BIT7);
          counter = 1;
         
        }
    }
  if (DiffValue > ThreshRange[2])
    {
      // Light up three LEDs  	        	
      if(temp == UP)                        // Tilt up 3, temp up 3
        {
          PJOUT |= BIT2 + BIT1 + BIT0;
          P3OUT &= ~(BIT7+BIT6+BIT5);
          counter = 7;
         

        }
      else                                  // Tilt down 3, temp down 3
        {
          P3OUT |= BIT5+BIT6+BIT7;
          PJOUT &= ~(BIT2+BIT1+BIT0);
          counter = 0;


        }
    }
    
}

/**********************************************************************//**
 * @brief  Transmit 7 bytes
 * 
 * @param  none 
 *  
 * @return none
 *************************************************************************/
void TXData()
{
  unsigned char counter = 0;
	
  // Configure UART 0
  UCA0CTL1 |= UCSWRST; 
  UCA0CTL1 = UCSSEL_2;                      // Set SMCLK as UCLk 
  UCA0BR0 = 52 ;                              // 9600 baud
  // 8000000/(9600*16) - INT(8000000/(9600*16))=0.083
  UCA0BR1 = 0; 
  // UCBRFx = 1, UCBRSx = 0x49, UCOS16 = 1 (Refer User Guide)
  UCA0MCTLW = 0x4911 ;                      
                                            
  UCA0CTL1 &= ~UCSWRST;                     // release from reset
  while(counter<7)
  {
    while (!(UCA0IFG&UCTXIFG));             // USCI_A0 TX buffer ready?
    UCA0TXBUF = TX_Buffer[counter];
    counter++;
  }	
}


void Mode3(void)
{ 
  
        
  // variable initialization
 // active = 1;    
  ADCTemp = 0;
  temp = 0;
  WriteCounter = 0;   
 // ULPBreakSync = 0;
  counter = 0;
  
  // One time setup and calibration
  SetupAccel();
  CalValue = CalibrateADC();
  while (1)
    {  
      // Take 1 ADC Sample
      TakeADCMeas();
      if (ADCResult >= CalValue)        
        {
          temp = DOWN;
          ADCTemp = ADCResult - CalValue;
        }
      else
        {
          temp = UP;
          ADCTemp = CalValue - ADCResult; 
        }                                       
      
          ULPBreakSync = 0;
          WriteCounter++;
          
          if(WriteCounter > 300)
            {
              angel=(float) atan2((ADCResult1-512),(ADCResult-512))*180/3.1415926;
              LEDSequence(angel,temp); 
              WriteCounter = 0;
              TX_Buffer[0]=0xFA;
              TX_Buffer[1]=angel;
              TX_Buffer[2]=0x00;
              TX_Buffer[3]=0x00;
              TX_Buffer[4]=0x00;
              TX_Buffer[5]=0x00; 
              TX_Buffer[6]=0XFE;
                
              TXData();    
              
           
            }
        
     }
    // end while() loop
    // turn off Accelerometer for low power
    ShutDownAccel();
}


 